"""
Various tests
"""

import pytest

# ----------------------
# is_on_pypi
# ----------------------

from mkdocs_macros.util import is_on_pypi  # Replace with actual import path

def test_known_package_exists():
    # requires connection
    assert is_on_pypi("requests", fail_silently=True) is True

def test_nonexistent_package():
    assert is_on_pypi("this_package_does_not_exist_123456", fail_silently=True) is False

def test_network_failure(monkeypatch):
    # Simulate network failure by patching requests.get to raise a RequestException
    import requests

    def mock_get(*args, **kwargs):
        raise requests.exceptions.ConnectionError("Simulated network failure")

    monkeypatch.setattr(requests, "get", mock_get)

    assert is_on_pypi("requests", fail_silently=True) is False

    with pytest.raises(RuntimeError):
        is_on_pypi("requests", fail_silently=False)



"""
Testing the project

(C) Laurent Franceschetti 2024
"""


import pytest

from mkdocs_test import DocProject





def test_build():
    project = DocProject(".")
    # did not fail

    print("building website...")
    build_result = project.build(strict=True)
    result = build_result.stderr
    print("Result:", result)
    # fails, declaring that the pluglet exists and must be installed.
    assert build_result.returncode != 0 # failure 
    assert "pluglet" in result
    assert "pip install" in result

    


"""
Testing the project

(C) Laurent Franceschetti 2025
"""
import pytest

from test.fixture import MacrosDocProject

CURRENT_PROJECT = '.'


def test_new_syntax():
    project = MacrosDocProject(CURRENT_PROJECT)
    project.build()
    # did not fail
    assert not project.build_result.returncode


    page = project.get_page('index')
    
    price = page.meta.unit_price
    print("Price found in the meta of the page:", price)
    assert page.find_text(f'{price} EUR')
    if price > 10:
        assert page.find_text('this is expensive')
    else:
        assert page.find_text('this is cheap')
        


"""
Specific for MkDocs Projects

(C) Laurent Franceschetti 2024
"""

import os
import warnings
import json
import subprocess
from typing import Dict


from super_collections import SuperDict
from mkdocs_test import DocProject, MkDocsPage
import os

class MacrosPage(MkDocsPage):
    "Specific for MkDocs-Macros"

    def has_error(self:MkDocsPage):
            "Predicate: check whether the page has an error"
            return self.find_text('Macro Rendering Error')
    
    @property
    def is_rendered(self):
        "Accomodate earlier formulation"
        warnings.warn("The page property `.is_rendered` is DEPRECATED "
                      "use `.is_markdown_rendered()` instead.", 
                      UserWarning, stacklevel=2)
        return self.is_markdown_rendered()

class MacrosDocProject(DocProject):
     "Specific for MkDocs-Macros"

     def build(self, strict:bool=False) -> subprocess.CompletedProcess:
          """
          Build the documentation, to perform the tests
          Verbose is forced to True, to get the variables, functions and filters
          """
          super().build(strict=strict, verbose=True)

     @property
     def pages(self) -> Dict[str, MacrosPage]:
          "List of pages"
          pages = super().pages
          return {key: MacrosPage(value) for key, value in pages.items()}
     

     
     @property
     def macros_plugin(self):
          "Information on the plugin"
          return self.get_plugin('macros')
     
     # ------------------------------------
     # Get information through the payload
     # ------------------------------------
     @property
     def variables(self):
               "Return the variables"
               try:
                    return self._variables
               except AttributeError:
                    print("ENTRIES:", self.find_entries("config variables",
                                                  source='',
                                                  severity='debug'))
                    print("ENTRIES:", self.find_entries("config variables",
                                                  source='macros'))
                    entry = self.find_entry("config variables",
                                                  source='macros',
                                                  severity='debug')
                    if entry and entry.payload:
                         payload = json.loads(entry.payload)
                         self._variables = SuperDict(payload)
                    else:
                         # print(entry)
                         # raise ValueError("Cannot find variables")
                         self._variables = {}
                    return self._variables


     @property
     def macros(self):
          "Return the macros"
          try:
               return self._macros
          except AttributeError:
               entry = self.find_entry("config macros",
                                        source='macros',
                                        severity='debug')
               if entry and entry.payload:
                    self._macros = SuperDict(json.loads(entry.payload))
               else:
                    # print(entry)
                    # raise ValueError("Cannot find macros")
                    self._macros = {}
               return self._macros
          

     @property
     def filters(self):
          "Return the filters"
          try:
               return self._filters
          except AttributeError:
               entry = self.find_entry("config filters",
                                             source='macros',
                                             severity='debug')
               if entry and entry.payload:
                    self._filters = SuperDict(json.loads(entry.payload))
               else:
                    #   print(entry)
                    #   raise ValueError("Cannot find filters")
                    self._filters = {}
               return self._filters
     
     # ------------------------------------
     # Special operations
     # ------------------------------------

     def add_file(self, pathname:str, content, binary=False) -> str:
          """
          Stores a file (any type) into the directory structure,
          relative to project directory.

          If you want to add a source page (markdown), prefer the
          add_source_page() method.

          Return the full pathname.
          """
          full_pathname = os.path.join(self.project_dir, pathname)
          # Ensure parent directory exists
          os.makedirs(os.path.dirname(full_pathname), exist_ok=True)
          if binary:
               with open(full_pathname, 'wb') as f:
                    f.write(content)
          else:
               with open(full_pathname, 'w', encoding="utf-8") as f:
                    f.write(content)
          return full_pathname


"""
Testing the module configurations 
"""

import os
import sys

import pytest



from .fixture import MacrosDocProject
from typing import List, Dict


# -------------------------------------
# Parameters
# -------------------------------------
DIR_NAME = '_temp'
UNIT_PRICE = 50
QUANTITY = 20
CONTENT =  f"""
                nav:
                    - Home: index.md

                extra:
                    unit_price: {UNIT_PRICE}
                    quantity: {QUANTITY}
                """


MODULE = '''
def define_env(env):
    """
    This is the hook for the variables, macros and filters.
    """

    @env.macro
    def price(unit_price, no):
        "Calculate price"
        return unit_price * no
'''

FIRST_PAGE = """
             # First page

             The total cost is {{ price(unit_price, quantity) }} euros.
             """


def prepare_site(site_name:str='My Test', 
                 plugins:List=['macros']) -> MacrosDocProject:
    "Basic content"
    if not isinstance(plugins, list):
        raise TypeError("plugins given are not a list")
    # create the path
    path = os.path.join(DIR_NAME, site_name).replace(' ', '_').lower()
    p = MacrosDocProject(path, new=True)
    p.clear()
    p.make_config(site_name=site_name,
                  content=CONTENT,
                  plugins = ['search', 'test'] + plugins
                  )
    return p


def config_with_module(name:str, module_position: str, config:Dict=None):
    """
    Create a site with module as directory 
    """
    if config is not None:
        p = prepare_site(site_name=name, plugins=[{'macros':config}])
    else:
        p = prepare_site(site_name=name)
    p.clear()
    p.add_source_page("index.md", FIRST_PAGE)
    p.add_file(module_position, MODULE)
    p.build(strict=True)
    assert p.success

    # first page
    page = p.get_page('index')
    assert page is not None, "Page not found"
    cost = UNIT_PRICE*QUANTITY
    assert page.find_text(f'{cost} euros'), f"Could not find expected cost ({cost}):\n{page.plain_text}" 



# -------------------------------------
# Tests with modules
# -------------------------------------

def test_module_simple():
    """
    Create a site with simple module
    """
    config_with_module('Simple', 'main.py')

def test_module_simple_other():
    """
    Create a site with simple module, different name
    """
    config_with_module('Simple Other', 'foo.py', config={'module_name': 'foo'})

def test_module_simple_other_sub():
    """
    Create a site with simple module, different name
    """
    config_with_module('Simple Other Sub', 'docs/macros/mymodule.py', 
                       config={'module_name': 'docs/macros/mymodule'})

def test_module_package():
    """
    Create a site with module as directory 
    """
    config_with_module('Package', 'main/__init__.py')

def test_module_package_other():
    """
    Create a site with module as directory 
    """
    config_with_module('Package Other', 'foo/__init__.py',
                       config={'module_name': 'foo'})

def test_module_package_other_error():
    """
    Create a site with module as directory, 
    with config error (bar instead of foo)
    """
    # Unfortunately MkdocsTest doesn't fail in error, if the build fails
    with pytest.raises((ImportError, AssertionError)):
        config_with_module('Package Other Error', 'foo/__init__.py', 
                           config={'module_name': 'bar'})

def test_module_package_other_sub():
    """
    Create a site with simple module, different name
    """
    config_with_module('Package Other Sub', 'docs/macros/mymodule/__init__.py',
                       config={'module_name': 'docs/macros/mymodule'})









   



def define_env(env):
    """
    This is the hook for the variables, macros and filters.
    """

    @env.macro
    def price(unit_price, no):
        "Calculate price"
        return unit_price * no
